# /packages/intranet-project-reminders/tcl/intranet-project-reminders-procs.tcl
#
# Copyright (C) 2013-now Project Open Business Solutions S.L.
#
# All rights reserved. Please check
# http://www.project-open.com/license/ for details.

ad_library {
    @author klaus.hofeditz@project-open.com

    # This is a very simplified implementation of a reminder system 
    # There's no tracking if email has been sent out 
    # Parameter LogP has been introduced to allow persistent logging into acs_logs
    # 
    # Extension  
    # 	Reminders or Projects / Sub-Projects 
    #   Custom timestamps for tasks and projects  
}

ad_proc -public im_project_reminders_send { } {
} {

    set offset_end_date_translation_tasks [parameter::get -package_id [apm_package_id_from_key intranet-project-reminders] -parameter "DefaultOffsetEndDateTranslationTasks" -default 3600]
    set offset_end_date_timesheet_tasks [parameter::get -package_id [apm_package_id_from_key intranet-project-reminders] -parameter "DefaultOffsetEndDateTimesheetTasks" -default 3600]
    set log_p [parameter::get -package_id [apm_package_id_from_key intranet-project-reminders] -parameter "LogP" -default 0]

    # Logging 
    if { $log_p } {
	if {[catch {
	    set sql "select acs_log__notice(
		'intranet-project-reminders::im_project_reminders_send',
		'Current/Last: [clock format [clock seconds] -format {%Y-%m-%d %H:%M:%S %Z}] / [db_string get_last_check "select last_check from im_project_reminders_last_check"]'
		)
	    "
	    set foo [db_string sql $sql]
	} err_msg]} {
	    global errorInfo
	    ns_log Error $errorInfo
	    set sql "
		select acs_log__error(
			'intranet-project-reminders::im_project_reminders_send',
			'Serious error fetching im_project_reminders_last_check::last_check: - please check error.log for more info'
		)
	    "
	    set foo [db_string sql $sql]
	}
    }

    # Setting "last_check"
    db_dml sql "update im_project_reminders_last_check set last_check = now()"

    # Find tasks
    set sql "
        select
                t.task_id as object_id,
		p.project_name as object_name,
		p.parent_id as parent_object_id,
		(select project_name from im_projects where project_id = p.parent_id) as parent_object_name,		
                p.end_date,
		(SELECT EXTRACT(EPOCH FROM (to_char(current_timestamp,'YYYY-MM-DD HH24:MI:00')::timestamp + 
			'$offset_end_date_timesheet_tasks seconds'::interval-to_char(p.end_date,'YYYY-MM-DD HH24:MI:00')::timestamp))) 
		as diff
        from
		im_timesheet_tasks t inner join im_projects p on t.task_id = p.project_id
	where 
		(SELECT EXTRACT(EPOCH FROM (to_char(current_timestamp,'YYYY-MM-DD HH24:MI:00')::timestamp +
			'$offset_end_date_timesheet_tasks seconds'::interval-to_char(p.end_date,'YYYY-MM-DD HH24:MI:00')::timestamp))) = 0
		and p.project_status_id not in ([join [im_sub_categories [im_project_status_closed]] ","])
    "

    if { [apm_package_enabled_p intranet-translation] } {
        append sql "
	UNION
        select
                tt.task_id as object_id,
		tt.task_name as object_name,
		tt.project_id as parent_object_id,
		(select project_name from im_projects where project_id = tt.project_id) as parent_object_name, 
                tt.end_date,
                (SELECT EXTRACT(EPOCH FROM (to_char(current_timestamp,'YYYY-MM-DD HH24:MI:00')::timestamp +
			'$offset_end_date_translation_tasks seconds'::interval-to_char(tt.end_date,'YYYY-MM-DD HH24:MI:00')::timestamp))) 
		as diff
        from
                im_trans_tasks tt
        where
		(SELECT EXTRACT(EPOCH FROM (to_char(current_timestamp,'YYYY-MM-DD HH24:MI:00')::timestamp +
			'$offset_end_date_translation_tasks seconds'::interval-to_char(tt.end_date,'YYYY-MM-DD HH24:MI:00')::timestamp))) = 0
		and task_status_id not in (360, 365, 370, 372)
    	"
    } 

    db_foreach r $sql {

	if { $log_p } {
	    if {[catch {
		set sql "select acs_log__notice(
                	'intranet-project-reminders::im_project_reminders_send', 
			'Calling im_project_reminders_send_notification_to_object_members for object_id: $object_id')
            	"
		set foo [db_string sql $sql]
	    } err_msg]} {
		global errorInfo
		ns_log Error $errorInfo
		set sql "
                select acs_log__error(
                        'intranet-project-reminders::im_project_reminders_send',
                        'Serious error fetching im_project_reminders_last_check::last_check: - please check error.log for more info')
            	"
		set foo [db_string sql $sql]
	    }
	}

	im_project_reminders_send_notification_to_object_members $object_id $object_name $parent_object_id $parent_object_name $diff "" 0

    }
}


ad_proc -private im_project_reminders_send_notification_to_object_members {
    { object_id "" }
    { object_name "" }
    { parent_object_id "" }
    { parent_object_name "" }
    { diff "" }
    { from_email "" }
    { do_not_send_email_p 0 }
} {
    Sends email reminders to supervisors
} {

    ns_log Notice "im_project_reminders_send_notification_to_object_members: Start Date: [clock format [clock seconds] -format {%Y-%m-%d-%H:%M:%S}]"
    ns_log Notice "im_project_reminders_send_notification_to_object_members: object_id: $object_id, object_name: $object_name, parent_object_id: $parent_object_id, parent_object_name: $parent_object_name, diff: $diff, from_email: $from_email"

    # #
    # Sanity check 

    if { "" eq $object_id } {
	ns_log Notice "im_project_reminders_send_notification_to_object_members: Exit - no object_id provided"
	return
    }

    # #
    # Defaults 

    set log_p [parameter::get -package_id [apm_package_id_from_key intranet-project-reminders] -parameter "LogP" -default 0]
    set object_type [acs_object_type $object_id]
    set parties_to_be_notified_id [list]
    set system_url [parameter::get -package_id [apm_package_id_from_key acs-kernel] -parameter "SystemURL" -default ""]

    if { "" eq $from_email } {
	set from_email [parameter::get -package_id [apm_package_id_from_key acs-kernel] -parameter "HostAdministrator" -default ""]
    }

    # #
    # Main    

    switch $object_type {
	"im_trans_task" {
	    	# Based on type we evaluate the mail recipient
		db_1row select_foo "
			select 
				tt.task_type_id,
				(select project_lead_id from im_projects where project_id = tt.project_id) as trans_project_lead_id,
				tt.trans_id,
				tt.edit_id,
				tt.proof_id,
				tt.other_id	
			from 
				im_trans_tasks tt
			where 
				tt.task_id = :object_id
		"
		
		# Get final responsible in workflow
		if { "" ne $trans_id } { lappend parties_to_be_notified_id $trans_id }

		if { "" ne $edit_id } { 
		    set parties_to_be_notified_id [list]
		    lappend parties_to_be_notified_id $edit_id
		}

		if { "" ne $proof_id } { 
		    set parties_to_be_notified_id [list]
		    lappend parties_to_be_notified_id $proof_id 
		}

		if { "" ne $other_id } { 
		    set parties_to_be_notified_id [list]
		    lappend parties_to_be_notified_id $other_id 
		}

		# Send always email to Project Manager
		lappend parties_to_be_notified_id $trans_project_lead_id

	}

	"im_timesheet_task" {
	    set object_name [db_string sql "select project_name from im_projects where project_id = :object_id" -default 0]
	    set sql "
        	    select
                	u.user_id
	            from
                	cc_users u,
	                acs_rels r
        	    where
                	r.rel_type = 'im_biz_object_member'
	                and r.object_id_one = :object_id
        	        and u.member_state = 'approved'
	                and u.user_id = r.object_id_two
        	    order by
                	u.user_id
    	    "
	    db_foreach r $sql {
		lappend parties_to_be_notified_id $user_id
	    }

	}
	default {
	    ns_log Notice "im_project_reminders_send_notification_to_object_members: Exit - object_type: $object_type not supported"
	    return
	}
    }

    # remove duplicates 
    set parties_to_be_notified_id [lsort -unique $parties_to_be_notified_id]

    # Set mail attributes 
    switch $object_type {
        "im_trans_task" {
	    set mail_body_txt "Planned end date of task:<br/>" 
            append mail_body_txt "&nbsp;&nbsp;&nbsp;&nbsp;<a href='${system_url}/intranet-translation/trans-tasks/task-list?project_id=$parent_object_id&view_name=trans_tasks'>$object_name</a><br/>"
            append mail_body_txt "of project: <br/>"
            append mail_body_txt "&nbsp;&nbsp;&nbsp;&nbsp;<a href='${system_url}/intranet/projects/view?project_id=$parent_object_id'>$parent_object_name</a><br/>"
	    append mail_body_txt "will be reached shortly and task has not been marked as DONE yet.<br/>Please review."	    
	}
        "im_timesheet_task" {
	    set mail_body_txt "Planned end date of task: <br/>"
            append mail_body_txt "&nbsp;&nbsp;&nbsp;&nbsp;<a href=${system_url}/intranet/projects/view?project_id=$object_id>$object_name</a><br/>"
	    append mail_body_txt "of project:<br/>"
            append mail_body_txt "&nbsp;&nbsp;&nbsp;&nbsp;<a href='${system_url}/intranet/projects/view?project_id=$parent_object_id'>$parent_object_name</a><br/>" 
            append mail_body_txt "will be reached shortly and task has not been marked as DONE yet.<br/>Please review."
	}
        default {
            ns_log Notice "im_project_reminders_send_notification_to_object_members: Exit - object_type: $object_type not supported"
            return
	}
    }

    set mail_subject [lang::message::lookup "" intranet-project-reminders.Mail_Subject_Text "Friendly Reminder - Task is due soon "]   

    set mail_body    "<html>"
    append mail_body "<head><meta charset=\"utf-8\"></head>"
    append mail_body "<body>"
    append mail_body [lang::message::lookup "" intranet-project-reminders.Mail_Body_Text_Html $mail_body_txt]
    append mail_body "</body>"
    append mail_body "</html>"

    if { $log_p } {
	if {[catch {
	    if { 0 == [llength $parties_to_be_notified_id] } {
		set sql "select acs_log__notice(
                        'intranet-project-reminders::im_project_reminders_send_notification_to_object_members',
                        'No parties to be notified found for object_id: $object_id')
                "
	    } else {
		set sql "select acs_log__notice(
                        'intranet-project-reminders::im_project_reminders_send_notification_to_object_members',
                        'Found following party/parties to be notified for object_id $object_id: $parties_to_be_notified_id')
                "
	    }

	    set foo [db_string sql $sql]

	} err_msg]} {
	    global errorInfo
	    ns_log Error $errorInfo
	    set sql "
                select acs_log__error(
                        'intranet-project-reminders::im_project_reminders_send',
                        'Serious error fetching im_project_reminders_last_check::last_check: - please check error.log for more info')
            "
	    set foo [db_string sql $sql]
	}
    }

    foreach email_recipient_id $parties_to_be_notified_id  {
	if { !$do_not_send_email_p } {
   	    set email_recipient [db_string sql "select email from parties where party_id = :email_recipient_id" -default 0]
	    acs_mail_lite::send \
		-send_immediately \
		-to_addr $email_recipient \
		-from_addr $from_email \
		-subject $mail_subject \
		-body $mail_body \
		-extraheaders "" \
		-mime_type "text/html"

	    ns_log Notice "im_project_reminders_send_notification_to_object_members: Sending email to: $email_recipient"
	} else {
	    ns_log Notice "im_project_reminders_send_notification_to_object_members: do_not_send_email_p = true -> NOT sending email to: $email_recipient"
	}
    }
    return 
}
